<?php

namespace WebsiteBundle\Command;

use Doctrine\ORM\EntityManager;
use Doctrine\ORM\EntityRepository;
use Symfony\Bundle\FrameworkBundle\Command\ContainerAwareCommand;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Output\OutputInterface;
use WebsiteBundle\Entity\Experiment;
use WebsiteBundle\Entity\TestSession;
use WebsiteBundle\Services\EmailService;

class SessionReminderCommand extends ContainerAwareCommand
{

  /**
   * @var EntityManager
   */
  private $entityManager;

  protected function configure()
  {
    $this
      ->setName('session:reminder')
      ->setDescription('Send a reminder email');
  }

  /**
   * @return EntityManager
   */
  private function getEntityManager() {
    if (empty($this->entityManager)) {
      $this->entityManager = $this->getContainer()->get('doctrine.orm.entity_manager');
    }

    return $this->entityManager;
  }

  /**
   * @param string $entityClass
   * @return EntityRepository
   */
  private function getRepository(string $entityClass) {
    if (empty($this->entityManager)) {
      $this->getEntityManager();
    }

    return $this->entityManager->getRepository($entityClass);
  }

  protected function execute(InputInterface $input, OutputInterface $output)
  {
    $output->writeln('<comment>Starting to retrieve participants who need to be sent a reminder email.</comment>');

    $emailService = $this->getContainer()->get('WebsiteBundle.services.EmailService');

    $experiments = $this->getCurrentExperiments();

    $output->writeln('<comment>'. count($experiments) .' experiments found. </comment>');

    foreach($experiments as $index => $experiment) {
      $output->writeln('<comment>Processing experiment id:'.$experiment['id'].'</comment>');

      if ($experiment['notifyBeforeSession'] === true) {
        $this->remindParticipantsBefore($output, $index, $emailService, $experiment);
      }

      if ($experiment['notifyAfterSession'] === true) {
        $this->remindParticipantsAfter($output, $index, $emailService, $experiment);
      }

      $this->entityManager->flush();
    }
  
    $output->writeln('<info>done</info>');
  }

  /**
   * @return array
   */
  private function getCurrentExperiments() {
    $experimentRepo = $this->getRepository(Experiment::class);
    return $experimentRepo->getCurrentExperiments();
  }

  /**
   * send an email to remind participants that their session will soon start.
   * @param OutputInterface $output
   * @param int $index
   * @param EmailService $emailService
   * @param array $experiment
   */
  private function remindParticipantsBefore($output, $index, $emailService, array $experiment) {
    $sessionRepo = $this->getRepository(TestSession::class);

    $notifyTime = $experiment['notificationTimeBeforeSession'];
    $sessions = $sessionRepo->getParticipantsThatNeedReminding($experiment['id'], $notifyTime);

    $output->writeln('<comment>processing experiment '.($index + 1).', '.count($sessions).' participants found to send before mail. </comment>');

    foreach($sessions as $session) {
      $emailService->sendReminderEmail($session);
      $session->setSentBeforeNotification(true);

      $this->entityManager->persist($session);
    }
  }

  /**
   * Send an email to remind participants that their session should already have started, but they most likely forgot!
   * @param OutputInterface $output
   * @param int $index
   * @param EmailService $emailService
   * @param array $experiment
   */
  private function remindParticipantsAfter($output, $index, $emailService, array $experiment) {
    $sessionRepo = $this->getRepository(TestSession::class);

    $notifyTime = $experiment['notificationTimeAfterSession'];
    $sessions = $sessionRepo->getParticipantsThatNeedReminding($experiment['id'], $notifyTime, 'after');

    $output->writeln('<comment>processing experiment '.($index + 1).', '.count($sessions).' participants found to send after mail. </comment>');

    foreach($sessions as $session) {
      $emailService->sendReminderEmail($session, 'after');
      $session->setSentAfterNotification(true);

      $this->entityManager->persist($session);
    }
  }

}
