<?php

namespace WebsiteBundle\Form\Admin;

use Bazookas\AdminBundle\Form\Type\NestedFormType;

use Doctrine\ORM\EntityManager;
use Symfony\Component\Form\AbstractType;
use Symfony\Component\Form\Extension\Core\Type\CheckboxType;
use Symfony\Component\Form\Extension\Core\Type\ChoiceType;
use Symfony\Component\Form\Extension\Core\Type\IntegerType;
use Symfony\Component\Form\Extension\Core\Type\TextType;
use Symfony\Component\Form\Extension\Core\Type\UrlType;
use Symfony\Component\Form\FormBuilderInterface;
use Symfony\Component\Form\FormInterface;
use Symfony\Component\Form\FormView;
use Symfony\Component\OptionsResolver\OptionsResolver;
use Symfony\Component\Validator\Constraints\Range;
use WebsiteBundle\Entity\Experiment;
use WebsiteBundle\Entity\Quote;

class ExperimentAdminType extends AbstractType
{
  private $entityManager;

  public function __construct(EntityManager $entityManager) {
    $this->entityManager = $entityManager;
  }

  /**
   * @param FormBuilderInterface $builder
   * @param array $options
   */
  public function buildForm(FormBuilderInterface $builder, array $options)
  {
    $experiment = $builder->getData();
    $experimentIsActive = !empty($experiment) ? $experiment->getSessionCount() > 0 : false;

    $builder
      //Panel 1
      ->add('name', TextType::class, [
        'label' => 'admin.entities.experiment.fields.name'
      ])
      ->add('experimentDuration', IntegerType::class, [
        'label' => 'admin.entities.experiment.fields.experimentDuration'
      ])

      //Panel 2
      ->add('conditions', NestedFormType::class,[
        'label' => false,
        'entry_type' => ExperimentConditionAdminType::class,
        'entry_options' => [
          'experiment' => $experiment,
          'surveyUrlToolTipBody' => $options['surveyUrlToolTipBody']
        ],
        'allow_delete' => !$experimentIsActive,
        'allow_add' => !$experimentIsActive,
        'prototype_name' => '__conditions__',
        'min' => 2
      ])

      //Panel 3
      ->add('totalSessions', TextType::class, [
        'label' => 'admin.entities.experiment.fields.totalSessions',
        'constraints' => new Range([
          'min' => 0,
          'max' => $this->calculateNumberOfQuotes(),
          'maxMessage' => 'admin.entities.experiment.maxSessionsByQuotes'
        ]),
        'disabled' => $experiment->getId() != null
      ])
      ->add('sessionDuration', IntegerType::class, [
        'label' => 'admin.entities.experiment.fields.sessionDuration'
      ])
      ->add('minTimeBetweenSession', IntegerType::class, [
        'label' => 'admin.entities.experiment.fields.minTimeBetweenSession'
      ])
      ->add('maxTimeBetweenSession', IntegerType::class, [
        'label' => 'admin.entities.experiment.fields.maxTimeBetweenSession'
      ])
      ->add('firstSessionLocation', ChoiceType::class, [
        'label' => 'admin.entities.experiment.fields.firstSessionLocation',
        'choices' => [
          'admin.entities.experiment.enums.LAST_SESSION_LAB' => Experiment::SESSION_LOCATION_LAB,
          'admin.entities.experiment.enums.LAST_SESSION_HOME' => Experiment::SESSION_LOCATION_HOME
        ],
        'expanded' => true
      ])
      ->add('lastSessionLocation', ChoiceType::class, [
        'label' => 'admin.entities.experiment.fields.lastSessionLocation',
        'choices' => [
          'admin.entities.experiment.enums.LAST_SESSION_LAB' => Experiment::SESSION_LOCATION_LAB,
          'admin.entities.experiment.enums.LAST_SESSION_HOME' => Experiment::SESSION_LOCATION_HOME
        ],
        'expanded' => true
      ])

      //Panel 4
      ->add('notifyBeforeSession', CheckboxType::class, [
        'label' => 'admin.entities.experiment.fields.notifyBeforeSession',
        'required' => false
      ])
      ->add('notificationTimeBeforeSession', IntegerType::class, [
        'label' => 'admin.entities.experiment.fields.notificationTimeBeforeSession',
        'required' => false,
        'conditionallyVisible' => [
          '#experiment_admin_notifyBeforeSession' => '$element.is(":checked")'
        ]
      ])

      ->add('notifyAfterSession', CheckboxType::class, [
        'label' => 'admin.entities.experiment.fields.notifyAfterSession',
        'required' => false
      ])
      ->add('notificationTimeAfterSession', IntegerType::class, [
        'label' => 'admin.entities.experiment.fields.notificationTimeAfterSession',
        'required' => false,
        'conditionallyVisible' => [
          '#experiment_admin_notifyAfterSession' => '$element.is(":checked")'
        ]
      ])

      //Panel 5
      ->add('psychEducationPanels', NestedFormType::class, [
        'label' => false,
        'entry_type' => PsychEducationPanelAdminType::class,
        'entry_options' => [
          'experiment' => $experiment
        ],
        'allow_delete' => !$experimentIsActive,
        'allow_add' => true,
        'prototype_name' => 'psych_panels'
      ])
      ->add('moodQuestions', NestedFormType::class, [
        'label' => false,
        'entry_type' => MoodQuestionsAdminType::class,
        'allow_delete' => !$experimentIsActive,
        'prototype_name' => 'mood_questions'
      ])
    ;
  }

  /**
   * @param OptionsResolver $resolver
   */
  public function configureOptions(OptionsResolver $resolver)
  {
    $resolver->setDefaults([
      'data_class' => Experiment::class,
      'renderType' => 'panel',  //TODO future version, allow the form to be rendered in different ways
      'grouping' => [
        'admin.entities.experiment.forms.panelHeader1' => [
          'name',
          'experimentDuration',
        ],
        'admin.entities.experiment.forms.panelHeader2' => [
          'conditions'
        ],
        'admin.entities.experiment.forms.panelHeader3' => [
          'totalSessions',
          'sessionDuration',
          'minTimeBetweenSession',
          'maxTimeBetweenSession',
          'firstSessionLocation',
          'lastSessionLocation'
        ],
        'admin.entities.experiment.forms.panelHeader4' => [
          'notifyBeforeSession',
          'notificationTimeBeforeSession',
          'notifyAfterSession',
          'notificationTimeAfterSession'
        ],
        'admin.entities.experiment.forms.panelHeader5' => [
          'psychEducationPanels'
        ],
        'admin.entities.experiment.forms.panelHeader6' => [
          'moodQuestions'
        ]
      ],
      'translation_domain' => 'admin',
      'surveyUrlToolTipBody' => ''
    ]);
  }

  public function buildView(FormView $view, FormInterface $form, array $options)
  {
    if (isset($options['grouping'])) {
      $view->vars['grouping'] = $options['grouping'];
    }
  }

  public function calculateNumberOfQuotes() {
    $quoteRepo = $this->entityManager->getRepository(Quote::class);
    return $quoteRepo->getQuoteCount();
  }

}
