<?php

namespace Bazookas\AdminBundle\Command\Notification;

use Bazookas\AdminBundle\Entity\Interfaces\Notification\EmailNotificationInterface;
use Bazookas\AdminBundle\Repository\Interfaces\Notification\EmailNotificationRepositoryInterface;
use Doctrine\ORM\EntityManager;
use Doctrine\ORM\EntityRepository;
use Doctrine\ORM\Mapping\MappingException;
use Symfony\Bundle\FrameworkBundle\Command\ContainerAwareCommand;
use Symfony\Bundle\FrameworkBundle\Templating\EngineInterface;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Output\OutputInterface;
use Symfony\Component\Routing\Router;
use Symfony\Component\Translation\TranslatorInterface;

class NotificationSendEmailCommand extends ContainerAwareCommand
{

  /**
   * @var EntityManager
   */
  private $entityManager;

  /**
   * @var string
   */
  private $emailNotificationClass;

  /**
   * @var EngineInterface
   */
  private $templateEngine;

  /**
   * @var \Swift_Mailer
   */
  private $mailer;

  /**
   * @var TranslatorInterface
   */
  private $translator;

  /**
   * @var Router
   */
  private $router;

  /**
   * @var string
   */
  private $fromEmail;

  protected function configure()
  {
    $this
      ->setName('notifications:email:send')
      ->setDescription('Send email notifications');
  }

  protected function execute(InputInterface $input, OutputInterface $output)
  {
    $this->initDependencies();

    $notificationRepo = $this->getRepository($this->getEmailNotificationClass());

    if (!$notificationRepo instanceof EmailNotificationRepositoryInterface) {
      throw new \Exception('Your repository must implement \'' . EmailNotificationRepositoryInterface::class . '\'');
    }

    $emailNotifications = $notificationRepo->getAllNotifications();

    if (empty($emailNotifications)) {
      $this->printOutput($output, 'Processed 0 notifications.');
      return;
    }

    //Sort the notifications by user so we can send 1 email for all of their notifications
    $sortedNotifications = $this->sortNotificationsByUser($emailNotifications);
    foreach ($sortedNotifications as $notifications) {
      $user = $notifications[0]['user'];
      $this->sendEmail($user, $notifications);
    }

    //Now remove the notifications
    $ids = $this->getNotificationIds($emailNotifications);
    $notificationRepo->removeNotifications($ids);

    $this->printOutput($output, 'Processed ' . count($emailNotifications) . ' notifications.');
  }

  private function initDependencies()
  {
    $container = $this->getContainer();

    $this->templateEngine = $container->get('templating');
    $this->router = $container->get('router');
    $this->mailer = $container->get('mailer');
    $this->translator = $container->get('translator');
    $this->entityManager = $container->get('doctrine.orm.entity_manager');

    $routerContext = $container->get('router')->getContext();
    $this->fromEmail = 'cms@' . $routerContext->getHost() . $routerContext->getBaseUrl();
  }

  /**
   * @param string $entityClass
   * @return EntityRepository
   */
  private function getRepository($entityClass)
  {
    if (empty($this->entityManager)) {
      $this->getEntityManager();
    }

    return $this->entityManager->getRepository($entityClass);
  }

  /**
   * @return EntityManager
   */
  private function getEntityManager()
  {
    if (empty($this->entityManager)) {
      $this->entityManager = $this->getContainer()->get('doctrine.orm.entity_manager');
    }

    return $this->entityManager;
  }

  /**
   * @return string
   * @throws \Exception
   */
  private function getEmailNotificationClass()
  {
    if (empty($this->emailNotificationClass)) {
      $notificationEntities = $this->getContainer()->getParameter('bazookas_admin.notifications.entities');
      $this->emailNotificationClass = $notificationEntities['EmailNotification'];

      try {
        $this->entityManager->getClassMetadata($this->emailNotificationClass);
      } catch (MappingException $e) {
        throw new \Exception('The defined email notification entity is not a registered doctrine entity');
      }
    }

    return $this->emailNotificationClass;
  }

  /**
   * @param OutputInterface $output
   * @param $message
   * @param string $type
   */
  private function printOutput(OutputInterface $output, $message, $type = 'comment')
  {
    $output->writeln('<' . $type . '>' . date('Y-m-d H:i:s') . ' - ' . $message . '</' . $type . '>');
  }

  /**
   * @param EmailNotificationInterface[] $notifications
   * @return array
   */
  private function sortNotificationsByUser(array $notifications)
  {
    $sortedNotifications = [];

    foreach ($notifications as $notification) {
      $userId = $notification['user']['id'];

      if (!isset($sortedNotifications[$userId])) {
        $sortedNotifications[$userId] = [];
      }

      $sortedNotifications[$userId][] = $notification;
    }

    return $sortedNotifications;
  }

  /**
   * @param array $user
   * @param array $notifications
   */
  private function sendEmail(array $user, array $notifications)
  {
    $this->translator->setLocale($user['locale']);

    $subject = $this->translator->trans('admin.notifications.email.subject', ['%count%' => count($notifications)], 'admin');
    $html = $this->templateEngine->render(
      '@BazookasAdmin/Emails/Notification/emailNotification.html.twig',
      [
        'user' => $user,
        'notifications' => $notifications,
        'notificationCount' => count($notifications)
      ]
    );

    $message = \Swift_Message::newInstance()
      ->setSubject($subject)
      ->setFrom($this->fromEmail)
      ->setTo($user['email'])
      ->addPart($html, 'text/html')
    ;

    $this->mailer->send($message);
  }

  /**
   * @param array $unsortedNotifications
   * @return array
   */
  private function getNotificationIds(array $unsortedNotifications)
  {
    $ids = [];

    foreach ($unsortedNotifications as $notification) {
      $ids[] = $notification['id'];
    }

    return $ids;
  }

}
