## Creating a basic dashboard

The CMS's AdminElements system allows you to easily create dashboards by building pages out of a variety of widgets.

Start by creating your controller. For your convenience a basic controller (`BaseAdminDashboardController`) class already exists for the creation of dashboard pages. You should therefore start by extending this base controller as shown in the example below.

```php
<?php

namespace DemoBundle\Controller\Admin;

use Bazookas\AdminBundle\Controller\Base\BaseAdminDashboardController;
use Bazookas\AdminBundle\PageBuilder\GenericPageBuilder;
use Bazookas\CommonBundle\Security\Roles;
use DemoBundle\Services\Dashboard\DemoDashboardService;
use Symfony\Component\HttpFoundation\Request;

class DemoDashboardController extends BaseAdminDashboardController
{

  protected function hasAccess($action) {
    return $this->isGranted(Roles::ROLE_CMS_USER);
  }

  public function modifyDashboardBuilder(Request $request, GenericPageBuilder $builder) {
    $dashboardService = new DemoDashboardService($this->get('translator'));
    $elements = $dashboardService->generateElements();

    foreach($elements as $element) {
      $builder->addElement($element);
    }

    return $builder;
  }

}
```

As with all cms controllers you must declare who has access to the page by implementing the `hasAccess` function.

Use the modify function for the dashboard action (default `modifyDashboardBuilder`) to add or remove elements to the page builder. In this case we've defined a separate service which is responsible for the actual creation of the widgets but there is nothing to stop you from adding elements directly in the `modifyDashboardBuilder` function.

So lets add a basic item to your page. Start by defining the function `generateSingleStatisticsRow` which will do the actual widget creation:

```php
  private function generateSingleStatisticsRow() {
    return new RowContainer([
      new StatisticElement([], [
        'value' => 5,
        'label' => 'demo.dashboard.statistics.item1',
        'icon' => 'fa-users',
        'panelColor' => 'info'
      ]),
      new StatisticElement([], [
        'value' => 81,
        'label' => 'demo.dashboard.statistics.item2',
        'icon' => 'fa-coffee',
        'panelColor' => 'primary'
      ]),
      new StatisticElement([], [
        'value' => '100%',
        'label' => 'demo.dashboard.statistics.item3',
        'icon' => 'fa-refresh',
        'panelColor' => 'success'
      ])
    ],[
      'defaultColClass' => 'col-md-4'
    ]);
  }
```

To add the new RowContainer to your page add the following to your `modifyDashboardBuilder` function.

```php
public function modifyDashboardBuilder(Request $request, GenericPageBuilder $builder) {
    $element = $this->generateSingleStatisticsRow();
    
    $builder->addElement($element);

    return $builder;
  }
```

That's it! You've now added a row of simple statistic widgets. Note that you can always look at the `DemoDashboardController` class in the DemoBundle in case you want to look at the example in more detail.
