<?php

namespace Bazookas\APIFrameworkBundle\Controller\Base;

use Bazookas\APIFrameworkBundle\Services\Data\Base\BaseDataService;
use Bazookas\CommonBundle\Utility\Utility;
use Bazookas\MediaBundle\Exception\MediaServiceException;
use Symfony\Bundle\FrameworkBundle\Controller\Controller;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;
use Bazookas\APIFrameworkBundle\Exception\InvalidArgumentException;
use Bazookas\APIFrameworkBundle\Exception\Interfaces\ApiExceptionInterface;
use Bazookas\APIFrameworkBundle\Exception\ServiceException;
use Symfony\Component\HttpFoundation\ParameterBag;
use Symfony\Component\HttpFoundation\Response;

//use status codes as constants from: http://api.symfony.com/3.0/Symfony/Component/HttpFoundation/Response.html

abstract class BaseController extends Controller
{
  const STATUS_SUCCESS = 'success';
  const STATUS_ERROR = 'error';

  /**
  * NOTE: Overwrite this function if you wish to use a specific data service instead of relying on the tagged data services
  *
  * @param string $dataServiceFunction the string name of the function to be executed in the data service
  * @return BaseDataService a data service
  */
  public function getDataService($dataServiceFunction) {
    $dataServiceChain = $this->get('APIFrameworkBundle.Data.DataServiceChain');

    $isDebug = $this->container->get('kernel')->isDebug();

    $dataService = null;
    foreach($dataServiceChain->getDataServices() as $service) {
      if (method_exists($service, $dataServiceFunction)) {
        if (empty($dataService)) {
          if (!$isDebug) {
            return $service;
          } else {
            $dataService = $service;
          }
        } else {
          throw new ServiceException("Wuuuttt?!! Two dataservices: ".get_class($dataService)." and ".get_class($service)." were found in the pool of tagged data services which contain the function '".$dataServiceFunction."'. Gods man! What do you expect me to do with this?!");
        }
      }
    }

    if (empty($dataService)) {
      //no service was found! *gasp* *horror* *gasp* throw the exception guv'nah!
      throw new ServiceException("No dataservice was found in the pool of tagged data services which contains the function '".$dataServiceFunction."'");
    } else {
      return $dataService;
    }
  }

  /**
  * @var Request request: the http request
  * @var array retrieveParams: an associative array of the parameters to be retrieved. The key of the assoc. array is the parameter name,
  * its value is the type of the
  * @return array
  */
  protected function getParameters(Request $request) {
    switch($request->getMethod()) {
      case 'GET':
        $parameterBag = 'query';
        break;
      default:
        $parameterBag = 'request';
    }

    $contentType = $request->headers->get('content-type');

    // preg_match returns int 1 on success
    switch(1) {
      case preg_match('#^[a-z]+/json#', $contentType):
        if ($request->getMethod() !== 'GET') {
          $bodyContent = json_decode($request->getContent(), true);
        } else {
          $bodyContent = $request->{$parameterBag}->all();
        }
        break;
      default:
        $bodyContent = $request->{$parameterBag}->all();
    }

    if (empty($bodyContent)) {
      return [];
    } else {
      return $bodyContent;
    }

  }

  /**
 * @var array retrieveParams: an associative array of the parameters to be retrieved. The key of the assoc. array is the parameter name,
 * its value is the type of the
 * @var array bodyContent: an array containing the parameters present in the request
 * @return ParameterBag a parameter bag containing the parameters
 */
  protected function validateParameters(array $retrieveParams, array $bodyContent) {
    $validationService = $this->get('APIFrameworkBundle.validation.validationService');
    return $validationService->validateData($retrieveParams, $bodyContent);
  }

  /**
   * @param array $data the data to be returned
   * @param int $statusCode an HTTP status code
   * @return JsonResponse
   */
  protected function sendResponse($data, $statusCode) {
    $response = new JsonResponse();

    if (isset($data)) {
      $response->setContent(json_encode($data));
    } else {
      $response->setContent("");
    }

    $response->setStatusCode($statusCode);
    return $response;
  }

  protected function errorResponse(ApiExceptionInterface $e) {
    $response = new JsonResponse();
    $response
      ->setData(
        array(
          'error' => $e->getName(),
          'error_description' => $e->getMessage()
        )
      )
      ->setStatusCode($e->getHttpStatusCode())
    ;

    return $response;
  }

  protected function handleRequest(Request $request, array $retrieveParams, $dataServiceFunction, $statusCode = Response::HTTP_OK, ParameterBag $extraParams = null) {
    try {
      $params = $this->getParameters($request);
      $params = $this->validateParameters($retrieveParams, $params);

      $dataService = $this->getDataService($dataServiceFunction);
      $data = call_user_func_array([$dataService, $dataServiceFunction], [$params, $extraParams]);

      return $this->sendResponse($data, $statusCode);
    }
    catch(ServiceException $e) {
       throw $e;
    }
    catch(ApiExceptionInterface $e) {
      return $this->errorResponse($e);
    }
    catch(MediaServiceException $e) {
      //catch and convert other specific error types
      $e = new InvalidArgumentException($e->getMessage(), $e->getCode(), $e);
      return $this->errorResponse($e);
    }
  }

}
