## Getting Started
### General introduction

The file upload service should be used when you wish to upload files through the api which should be linked to a specific entity. An example of this might be the avatar image that should be associated with a user entity. The api file upload service will handle the file upload logic and aims to become a unified method by which file upload will be handled.

The file upload flow will always be:
1. Send entity meta data (POST api call)
2. Return upload url(s)
3. Send file(s) to returned url

Note that uploads must be initiated through a prior API call first. At the time of this writing the API does not provide a way to upload files directly.

### 1. Setting up your controller

Start by creating a standard Api controller. For more information on how to create an api controller and how to create data services please see the relevant [documentation](Resources/docs/apiController.md). 

For brevity a sample implementation is shown below:
```php
  public function updateTeamAction(Request $request) {
    $params = [
      'presetImage' => [
        'constraint' => [
          new Type('int'),
        ],
      ],
    ];

    $extraParams = new ParameterBag(array(
      'user' => $this->getUser()
    ));

    return $this->handleRequest($request, $params, 'updateTeam', Response::HTTP_OK, $extraParams);
  }
```

### 2. Defining your data service

Below is a sample implementation of your `updateTeam` data service function (this continues the example shown in the earlier section):
```php
  public function updateTeam(ParameterBag $params, ParameterBag $extraParams) {

     //Do stuff

    $result = $this->handleImageUpload($params, $team);
    $result['teamId'] = $team->getId();

    return $result;
  }
```

The `handleImageUpload` function might look as follows:
```php

  protected function handleImageUpload($params, $image) {
      $uploadTargets = $this->uploadService->createFileUploads([
        [
          'targetDirectory' => 'images/profile',
          'overwrite' => true,
          'filename' => 'image_' . $image->getId(),
          'mimeTypes' => '#^image/.+$#',
          'callback' => new APIFileUploadCallback(
            'BazookasMediaBundle.dataService.APICallBackService',
            'uploadComplete',
            ['id' => $image->getId()]
          ),
        ],
      ],
        new APIFileUploadCallback(
          'apibundle.services.CacheManager',
          'refreshCache',
          ['relativeUrl' => $image->getRelativeUrl()]
        )
      );

      $result['uploadTarget'] = $uploadTargets[0];

    return $result;
  }
```

The file upload service requires that you implement a file upload configuration as shown below:
```php
$uploadTargets = $this->uploadService->createFileUploads([
[
  'targetDirectory' => 'images/profile',
  'overwrite' => true,
  'filename' => $fileNamePrefix . $entity->getId(),
  'mimeTypes' => '#^image/.+$#',
  'callback' => new APIFileUploadCallback(
    'BazookasMediaBundle.dataService.APICallBackService',
    'uploadComplete',
    ['id' => $image->getId()]
  ),
],
],
new APIFileUploadCallback(
  'apibundle.services.CacheManager',
  'refreshCache',
  ['relativeUrl' => $image->getRelativeUrl()]
)
);

$result['uploadTarget'] = $uploadTargets[0];
```

In the above code you define an api file upload service callback which will update the given image entity with the file uploaded by the user to the given upload target. The upload target is a url which is to be used by the user to upload their image, each upload target is unique. The upload targets returned by the api file upload service will look as follows:

```php
[
  0 => 'http://localhost:8000/api/v1.0/files/upload/5op79qk7z3k88gw4gccw44sw8k8wws408go44kgkc8sw04k4go/0'
]

```

Note that the index matches the index provided in the file uploads service. The index is also the last parameter in the route. This means that each file upload call is able to link multiple images to one or more entities.
