<?php

namespace Bazookas\APIFrameworkBundle\Services\Upload;

use Bazookas\APIFrameworkBundle\Entity\APIFileUpload;
use Bazookas\APIFrameworkBundle\Exception\ApiException;
use Bazookas\APIFrameworkBundle\Exception\Base\ApiExceptionMap;
use Bazookas\APIFrameworkBundle\Exception\ElementNotFoundException;
use Bazookas\APIFrameworkBundle\Exception\InvalidArgumentException;
use Bazookas\APIFrameworkBundle\Services\Data\Base\BaseDataService;
use Bazookas\APIFrameworkBundle\Services\Data\Interfaces\DataServiceInterface;
use Bazookas\APIFrameworkBundle\Util\APIFileUploadCallback;
use Doctrine\ORM\EntityManager;
use Symfony\Bundle\FrameworkBundle\Routing\Router;
use Symfony\Component\DependencyInjection\ContainerAwareInterface;
use Symfony\Component\DependencyInjection\ContainerAwareTrait;
use Symfony\Component\Filesystem\Exception\IOException;
use Symfony\Component\Filesystem\Filesystem;
use Symfony\Component\HttpFoundation\ParameterBag;
use Symfony\Component\Intl\Exception\NotImplementedException;

class APIFileUploadService extends BaseDataService implements ContainerAwareInterface
{
  use ContainerAwareTrait;

  /**
   * @var EntityManager
   */
  private $entityManager;

  /**
   * @var Router
   */
  private $router;

  /**
   * @var string
   */
  private $webDirectory;
  private $baseUrl;
  private $uploadsDirectory;
  private $uploadRoute;
  private $allowOctetStream = false;

  public function __construct(EntityManager $em, Router $router, $settings)
  {
    $this->entityManager = $em;
    $this->repository = $em->getRepository('BazookasAPIFrameworkBundle:APIFileUpload');
    $this->router = $router;
    $this->allowOctetStream = $settings['allow_application_octet_stream'];

    $this->webDirectory = realpath($settings['root_directory']);
    // when base URL is empty, get the default base URL from the router
    if (empty($settings['baseUrl'])) {
      $context = $router->getContext();
      $this->baseUrl =
        $context->getScheme().'://'.
        $context->getHost();
      ;
      if ($context->getScheme() === 'http') {
        if ($context->getHttpPort() != 80) {
          $this->baseUrl .= ':'.$context->getHttpPort();
        }
      }
      $this->baseUrl .= $context->getBaseUrl();
    }
    else {
      $this->baseUrl = $settings['baseUrl'];
    }
    $this->uploadsDirectory = realpath($settings['uploads_root_directory']);
    $this->uploadRoute = $settings['uploadRoute'];
  }

  /**
   * Examples in Documentation/apiFileUpload.md
   *
   * @param $files array of files to be uploaded, note that the order of keys does not really matter as long as they are numeric
   * @param APIFileUploadCallback|null $allFilesComplete
   * @return array
   */
  public function createFileUploads($files, APIFileUploadCallback $allFilesComplete = null) {
    $hash = md5(uniqid());

    if (isset($allFilesComplete)) {
      $this->checkIfServiceExists($allFilesComplete);
    }

    $em = $this->entityManager;
    $result = array();

    foreach ($files as $index => $file) {
      $fileUpload = new APIFileUpload();
      $fileParams = new ParameterBag($file);

      // TODO set restrictions to content type?
      $fileUpload
        ->setHash($hash)
        ->setUploadIndex($index)
        ->setTargetDirectory($file['targetDirectory']) // don't use ParameterBag because this is required
        ->setPreferredFileName($fileParams->get('filename'))
        ->setOnUploadCallback($fileParams->get('callback'))
        ->setOverwrite($fileParams->get('overwrite', false))
        ->setMimeTypes($fileParams->get('mimeTypes'))
        ->setOnCompleteCallback($allFilesComplete)
      ;

      $em->persist($fileUpload);

      $result[$index] = $this->router->generate($this->uploadRoute, array(
        '_hash' => $hash,
        '_index' => $index,
      ), Router::ABSOLUTE_URL);
    }

    $em->flush();

    return $result;
  }

  /**
   * @param $hash
   * @param $index
   * @param $contentType
   * @param $fileName
   * @param $file
   * @return mixed
   */
  public function uploadFullFile(ParameterBag $params, ParameterBag $extraParams) {
    $contentType = $extraParams->get('contentType');
    $contentLength = $extraParams->get('contentLength');
    $fileName = $extraParams->get('fileName');
    /** @var resource $file */
    $file = $extraParams->get('file');
    // TODO find a better way for this?? We could use the "$asResource" param in FileUploadController, but then file size does not match
    $mimeType = mime_content_type(fopen('php://input', 'rb'));
    $fileSize = strlen($file);

    $index = $extraParams->get('index');
    $hash = $extraParams->get('hash');

    if (empty($contentType)) {
      throw new InvalidArgumentException('\'Content-Type\': This header is required', ApiExceptionMap::MISSING_HEADER);
    }
    else if ($contentType !== $mimeType) {
      if ($this->allowOctetStream && $mimeType === 'application/octet-stream') {
        $mimeType = $contentType;
      }
      else {
        throw new InvalidArgumentException('\'Content-Type\': ' . $contentType . ' not match file mime type ('.$mimeType.')', ApiExceptionMap::INVALID_CONTENT_TYPE);
      }
    }
    if ($fileSize === 0) {
      throw new InvalidArgumentException('\'Content:\' Empty post body is not allowed', ApiExceptionMap::CONTENT_CANNOT_BE_EMPTY);
    }
    else if ($fileSize !== $contentLength) {
      throw new InvalidArgumentException('\'Content-Length\': header does not match the length of the post body', ApiExceptionMap::CONTENT_LENGTH_DOES_NOT_MATCH);
    }

    $uploads = $this->repository->findBy(array(
      'hash' => $hash,
//      'uploadIndex' => $index, // NOTE don't send index to be able to check if it is the last one!
//      'completed' => false,
    ));

    /* @var APIFileUpload $targetUpload */
    $targetUpload = null;
    $incompleteFileCount = 0;

    foreach ($uploads as $upload) {
      // no strict equality to allow strings
      if ($upload->getUploadIndex() == $index) {
        // FOUND IT
        $targetUpload = $upload;
      }
      // increment the incomplete file count, but don't count the current file
      else if (!$upload->isCompleted()) {
        $incompleteFileCount++;
      }
    }

    if (empty($targetUpload)) {
      throw new ElementNotFoundException('File upload not found');
    }
    if ($targetUpload->isCompleted()) {
      throw new InvalidArgumentException('File already uploaded', ApiExceptionMap::DUPLICATE_ENTRY);
    }

    if (!preg_match($targetUpload->getMimeTypes(), $mimeType)) {
      throw new InvalidArgumentException('\'Content-Type\': Only /'.$targetUpload->getMimeTypes().'/ are allowed', ApiExceptionMap::INVALID_CONTENT_TYPE);
    }

    $this->moveFile($targetUpload, $fileName, $contentType, $file);

    // we got this far; which can only mean success!!
    $targetUpload->setCompleted(true);

    $this->executeCallback($targetUpload->getOnUploadCallback(), $targetUpload);

    // and if it is the final upload, also execute the onComplete callback
    if ($incompleteFileCount === 0) {
      $this->executeCallback($targetUpload->getOnCompleteCallback(), $targetUpload);
    }

    $this->entityManager->persist($targetUpload);
    $this->entityManager->flush();

    // get the final url and return it
    $fileUrl = $targetUpload->getFinalUrl();
    return [
      'data' => $fileUrl,
    ];
  }

  /**
   * @param APIFileUpload $targetUpload
   * @param $fileName
   * @param $contentType
   * @param $file
   * @throws IOException
   */
  private function moveFile(APIFileUpload $targetUpload, $fileName, $contentType, $file) {
    $fileName = $targetUpload->getPreferredFileName($fileName);

    if (empty($fileName)) {
      $fileName = 'api_file_upload';
      // prevent overwriting when it is a generated name
      $targetUpload->setOverwrite(false);
    }

    $fileName = explode('.', $fileName);
    if (count($fileName) > 1) {
      $extension = '.' . array_pop($fileName);
    }
    else {
      $extension = '.' . $this->getExtensionByMimeType($contentType);
    }
    $fileName = implode('.', $fileName);
    $fileSuffix = '';
    $fileSuffixNumber = 0;

    $fs = new Filesystem();
    $targetDirectory = $this->uploadsDirectory . '/' . $targetUpload->getTargetDirectory();

    // in case it doesn't exist, blow it up again (erm... create a directory)
    $fs->mkdir($targetDirectory);

    // prepend the targetDirectory to the filename so we don't need to do it constantly
    $fileName = $targetDirectory . '/' . $fileName;

    // create a unique file name if overwrite is false
    if (!$targetUpload->isOverwrite()) {
      // check if file exists
      while ($fs->exists($fileName.$fileSuffix.$extension)) {
        $fileSuffixNumber++;
        $fileSuffix = "_$fileSuffixNumber";
      }
    }

    $fileName = $fileName.$fileSuffix.$extension;
    $fs->dumpFile($fileName, $file);

    // set the final file name
    $finalPath = str_replace($this->webDirectory, '', $fileName);
    $targetUpload
      ->setFile($fileName)
      ->setRelativeUrl($finalPath)
      ->setFinalUrl($this->baseUrl . $finalPath)
    ;
  }

  /**
   * Execute a callback
   * @param APIFileUploadCallback|null $callback if null, nothing happens
   */
  private function executeCallback(APIFileUploadCallback $callback = null, $targetUpload) {
    if (isset($callback)) {
      $service = $this->container->get($callback->getService());
      $method = $callback->getMethod();
      if (method_exists($service, $method)) {
        $parameterBag = $callback->getParameterBag();
        $parameterBag->set('APIFileUpload', $targetUpload);
        call_user_func(array($service, $method), $parameterBag);
      }
    }
  }

  private function getExtensionByMimeType($mimeType) {
    // TODO make decent extension mapping!
    // use this lib? https://github.com/dflydev/dflydev-apache-mime-types
    switch ($mimeType) {
      case 'image/jpg':
      case 'image/pjpeg':
      case 'image/jpeg': return 'jpg';
      case 'image/png': return 'png';
      case 'image/gif': return 'gif';
      case 'image/tiff': return 'tiff';
      case 'image/svg':
      case 'image/svg+xml': return 'svg';
      default:
        return array_pop(explode('/', $mimeType));
    }
  }

  public function writeFilePart() {
    throw new NotImplementedException('this method is not implemented yet');
  }

  private function checkIfServiceExists(APIFileUploadCallback $callback) {
    $service = $callback->getService();

    if (!$this->container->has($service)) {
      throw new ApiException('Upload callback service does not exist: '.$service);
    }
  }
}
