<?php

namespace Bazookas\CommonBundle\Command\Deploy;

use Symfony\Bundle\FrameworkBundle\Command\ContainerAwareCommand;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Output\OutputInterface;
use Symfony\Component\Filesystem\LockHandler;
use Sensio\Bundle\DistributionBundle\Composer\ScriptHandler;
use Symfony\Component\Process\Process;
use Symfony\Component\Process\Exception\ProcessFailedException;
use Symfony\Component\Filesystem\Filesystem;
use Symfony\Component\Console\Input\ArrayInput;

class PostDeployCommand extends ContainerAwareCommand
{
  private $command = 'project:deploy:post';

  protected function configure()
  {
    $this
      ->setName($this->command)
      ->setDescription("Runs post deployment optimizations")
    ;
  }

  protected function execute(InputInterface $input, OutputInterface $output)
  {
    $output->writeln("<comment>".date('Y-m-d H:i:s')." - Starting post deploy optimizations...</comment>");

    //clear the production cache
    $this->clearCache($output);

    //set the folder permissions
    $this->setVarFolderPermissions($output);

    //Optimize the composer autoloader
    $this->optimizeAutoloader($output);

    $output->writeln("<info>".date('Y-m-d H:i:s')." - Done</info>");
  }

  private function clearCache(OutputInterface $output) {
    $output->writeln("<comment>".date('Y-m-d H:i:s')." - Running cache clear...</comment>");

    $command = $this->getApplication()->find('cache:clear');
    $commandInput = new ArrayInput([
      'command' => 'cache:clear',
      '--env' => 'prod',
      '--no-debug' => true,
      '--no-warmup' => true,
    ]);
    $command->run($commandInput, $output);
  }

  private function setVarFolderPermissions(OutputInterface $output) {
    $output->writeln("<comment>".date('Y-m-d H:i:s')." - Setting folder permissions and owner...</comment>");

    $appRootDir = $this->getContainer()->getParameter('kernel.root_dir');

    $fs = new Filesystem();
    // symfony caches, logs, etc...
    $this->createWritableFolder($output, $fs, $appRootDir, 'var');
    // bazookas uploads location
    $this->createWritableFolder($output, $fs, $appRootDir, 'web/uploads');
    // default imagine cache location, can be overwritten in config !! (using `cache_prefix` in imagine resolver config)
    // (cms sets to web/uploads/cache, which is fine because of the above)
    $this->createWritableFolder($output, $fs, $appRootDir, 'web/media');

    $output->writeln("<info>".date('Y-m-d H:i:s')." - Changed file permissions.</info>");
  }

  private function createWritableFolder(OutputInterface $output, Filesystem $fs, $appRootDir, $folder) {
    if (!$fs->exists($appRootDir.'/../'.$folder)) {
      $output->writeln("<info>".date('Y-m-d H:i:s')." - Creating `$folder` folder...</info>");
      $fs->mkdir($appRootDir.'/../'.$folder, 0775);

      // set permissions
      $process = new Process(
        'HTTPDUSER=`ps axo user,comm | grep -E \'[a]pache|[h]ttpd|[_]www|[w]ww-data|[n]ginx\' | grep -v root | head -1 | cut -d\  -f1`'.
        ' && '.
        'sudo setfacl -R -m u:"$HTTPDUSER":rwX -m u:`whoami`:rwX '.$folder.
        ' && '.
        'sudo setfacl -dR -m u:"$HTTPDUSER":rwX -m u:`whoami`:rwX '.$folder
      );
      $process->run();
    }
  }

  private function optimizeAutoloader(OutputInterface $output) {
    $output->writeln("<comment>".date('Y-m-d H:i:s')." - Running composer optimize autoloader...</comment>");

    $command = '';

    $appRootDir = $this->getContainer()->getParameter('kernel.root_dir');
    $fs = new Filesystem();
    if ($fs->exists($appRootDir.'/../composer.phar')) {
      $command = $appRootDir.'/../composer.phar dump-autoload --optimize --no-dev --classmap-authoritative';
    } else {
      $command = 'composer dump-autoload --optimize --no-dev --classmap-authoritative';
    }

    $output->writeln("<comment>".date('Y-m-d H:i:s')." - Attempting command '".$command."'</comment>");

    $process = new Process($command);
    $process->run();

    // executes after the command finishes
    if (!$process->isSuccessful()) {
      $output->writeln("<error>".$process->getErrorOutput()."</error>");
    }
    $output->writeln("<info>".$process->getOutput()."</info>");
  }

}
