## Enums

This bundle provides a `BaseEnum` class which you can use to define your constants. Note that, by Bazookas convention, the enums should be defined in a folder `src\WebsiteBundle\Enums` where `WebsiteBundle` is your project specific bundle.

An example:

```php
<?php

namespace ApiBundle\Enums;

use Bazookas\CommonBundle\Enums\Base\BaseEnum;

class Province extends BaseEnum
{

  const WEST_FLANDERS = 0;
  const EAST_FLANDERS = 1;
  const ANTWERP = 2;
  const FLEMISH_BRABANT = 3;
  const LIMBURG = 4;

}
``` 

The reason why you would use the `BaseEnum` class is because it provides the following convenience functions:

- `BaseEnum::getConstants()` get all of the constants defined in the class
- `BaseEnum::isValidName($name)` check if the given string is the name of a constant
- `BaseEnum::isValidValue($value)` check if the given value is mapped to a constant
- `BaseEnum::getNameForValue($value)` return the name of the constant for the given value

**NOTE**: A class extending BaseEnum can never be instantiated. For example, attempting to do `new FoodCategory()` will result in a fatal error.

### Validating an enum constraint

This bundle also provides an `EnumConstraint` which allows you to use the symfony validation component or the APIFramework validation service to validate if the given value is a defined enum value.

Example usage:

```php
<?php

namespace ApiBundle\Controller\v1_0;

use ApiBundle\Enums\Province;
use Bazookas\CommonBundle\Validator\Constraints\EnumConstraint;
use Symfony\Component\HttpFoundation\Request;
use Bazookas\APIFrameworkBundle\Controller\Base\BaseController ;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpFoundation\ParameterBag;

class UserController extends BaseController
{
  public function createUserAction(Request $request) {
    $retrieveParams = [
      'province' => [
        'constraint' => new EnumConstraint([
          'constraintClass' => Province::class
        ]),
        'required' => true
      ]
    ];

    $extraParams = new ParameterBag([
      'locale' => $request->getLocale(),
      'user' => $this->getUser()
    ]);
    
    return $this->handleRequest($request, $retrieveParams, 'createUser', Response::HTTP_OK, $extraParams);
  }
}  
```

Note that it is also possible to use the constraint in your entity in the same way as you would use a normal constraint:

```php
<?php

namespace ApiBundle\Entity;

use Bazookas\CommonBundle\Validator\Constraints\EnumConstraint;
use Doctrine\ORM\Mapping as ORM;
use Bazookas\CommonBundle\Entity\Base\BaseUser;
use Bazookas\CommonBundle\Entity\Traits;
use Bazookas\CommonBundle\Entity\Interfaces\LocalisedEntityInterface;

/**
 * @ORM\Table(name="app_user")
 * @ORM\Entity
 * @ORM\HasLifecycleCallbacks
 */
class AppUser extends BaseUser implements LocalisedEntityInterface
{

  use Traits\LocalisedTrait;

  /**
   * @ORM\Column(type="string")
   * @var string
   * @EnumConstraint(constraintClass="ApiBundle\Enums\Province")
   */
  private $province;

}

```
