## The Export-configuration

When using the export-service, you need to provide it with an export-configuration in order for it to work.

A sample configuration is provided below:
```php
$exportConfig = [
    'sheets' => [
      [
        'entity' => $this->config->entity,
        'fields' => [
          'id' => 'Id'
          'name' => 'admin.entities.park.fields.name'
          'weight' => 'admin.entities.park.fields.weight'
          'parkElements.id' => 'admin.entities.parkElement.fields.name',
          'parkElements.details.locale' => 'admin.entities.parkElementDetails.fields.locale'
          'parkElements.details.name' => 'admin.entities.parkElementDetails.fields.name'
        ],
        'groupBy' => 'e_parkElements_details.id',
        'conditions' => [
          'e.id = :id',
        ],
        'parameters' => [
          'id' => $this->id,
        ],
        'sheetName' => 'Elements belonging to park with id ' . $this->id,
      ],
      [
        ...
      ],
    ],
    'extraParams' => [
      ...
    ],
    'fileName' => 'Park-element-export',
    'format' => ExportService::TYPE_XLSX,
    'dataService' => 'defaultService',
    'renderSync' => true,
    'user' => $this->user->getEmail(),
];    
```
The export-configuration contains several elements:
* `sheets`: This contains an array with all the information necessary to select data for one (or more) sheet(s) in an export. When exporting to .csv format, only the first element in the array will be used, as CSV's can only contain one sheet. When exporting to .xlsx format, there will be a sheet for each element in the array.
    * `entity`: You need to provide the base-entity to select data from. In the sample configuration, we are using the entity related to the controller where we are calling the export-function, but you can also provide an entity like this: `Park::class`.
    * `fields`: This array contains all the fields you want to export from the database. The keys of the array are the fields of the database, while the value is the text that will be displayed at the top of the column in the export. You can use the translation files for this text, or you can provide your own custom text.
    * `groupBy (optional)`: If you want to group your data by a certain field, this is the way to do it.
    * `conditions (optional)`: If you provide no conditions, the service will fetch all the data from the requested fields. Otherwise, you can add several conditions to the configuration. When adding conditions to your export, you have to use parameters where necessary, and provide the parameters.
    * `parameters (optional)`: When adding conditions to your export-configuration, you provide the relevant parameters here.
    * `sheetName (optional)`: When exporting to xlsx format, you can provide a name for the sheet to use. If no sheetname is provided, the service will use Sheet 1, Sheet 2, Sheet 3, ...
* `extraParams (optional)`: If you are using your own service to select data from the database, it is possible that you need other parameters than usual. More about this here:

[How to use your own service for creating the data array](dataService.md)
* `fileName`: The name that you want to give to your file. Please note that your file will only have this exact name when making synchronous exports, as asynchronous exports get random text added to their filename to prevent duplicate file names from overwriting each other.
* `format`: The format that you want for your export. Options right now are .csv or .xlsx.
* `dataService`: The name of the service that you want to use to select data from the database. If you want to use the default service, use "defaultService" like in the example configuration.
* `renderSync`: A boolean that determines if the export is made synchronous or asynchronous. When making synchronous exports, the user gets their file as soon as the server is finished with selecting the data and creating the file. However, if you need to select a lot of data, this can take a while and leave the user waiting. In that case, it is recommended to use the asynchronous option, in which case the user will receive an email with the download link to their file. Set true for synchronous, false for asynchronous.
* `user`: Provide the email-address of the user to send the email to when using asynchronous exports.

#### Conditions

In the example configuration above, we provided a condition in the following way:

```php
'conditions' => [
  'e.id = :id',
]
```
When there are multiple requirements that need to be fulfilled, each requirement can be an element in the array:
```php
'conditions' => [
  'e.id = :id',
  'e_options_answers_user_profile.gender like :gender',
]
```
As can be seen in the example above, we provided a requirement for the base entity and a requirement for a related entity.
When we have a requirement for a field of the base entity, we refer to it in the following way:
```php
'e.{field} = :{field}'
```
where you replace {field} with the relevant field.

When we have a requirement for a field of a related entity, we refer to it in the following way:
```php
'e_{related}_{related2}.{field} = :{field}
```
where {related} is  an entity related to the base entity, {related2} is an entity related to {related} and {field} is a field belonging to entity {related2}.

In the previous examples, the data had to fulfill all the provided requirements. In some cases, the data must fulfill one OR another requirement.
This can be done in the following way:
```php
'conditions' => [
  new \Doctrine\ORM\Query\Expr\Orx(['e_options_answers_user_profile.gender like :gender', 'e_options_answers_user_profile.email IS NOT NULL']),
]
```
In this example, the service will only export the data where the user is the specified gender or if their email is not null.

