## Using the export service

When using the export service, there are 2 steps that need to be completed.

1. Create a valid export config
2. Call the export service with the config

### Example
Here is an example of how you can use the export service:
```php
/**
   * Export individual answers per session
   * @param Request $request
   * @param $action
   * @return \Symfony\Component\HttpFoundation\Response
   * @throws \Exception
   */
  protected function exportAnswerAction(Request $request, $action) {
    $sessionId = $request->query->get('sessionId');

    $entityManager = $this->get('doctrine.orm.entity_manager');

    $sessionRepo = $entityManager->getRepository(TestSession::class);
    $participantId = $sessionRepo->getParticipantId($sessionId);
    $session = $sessionRepo->findOneById($sessionId);

    $answerRepo = $entityManager->getRepository(TestAnswer::class);

    $config = [
      'sheets' => [
        [
          'entity' => TestAnswer::class,
          'fields' => [
            'participant.id' => 'admin.entities.participant.fields.id',
            'session.number' => 'admin.entities.testSession.fields.number',
            'trial' => 'admin.entities.testAnswer.fields.trial',
            'stimulus' => 'admin.entities.testAnswer.fields.stimulus',
            'answer' => 'admin.entities.testAnswer.fields.answer',
            'correctAnswer' => 'admin.entities.testAnswer.fields.correctAnswer',
            'correct' => 'admin.entities.testAnswer.fields.correct',
            'reactionTime' => 'admin.entities.testAnswer.fields.reactionTime',
            'interval' => 'admin.entities.testAnswer.fields.interval',
            'consecutiveCorrect' => 'admin.entities.testAnswer.fields.consecutiveCorrect',
            'consecutiveWrong' => 'admin.entities.testAnswer.fields.consecutiveWrong',
            'level' => 'admin.entities.testAnswer.fields.level',
            'score' => 'admin.entities.testAnswer.fields.score',
          ],
          'queryBuilder' => $answerRepo->createAnswerExportQuery($sessionId)
        ]
      ],
      'fileName' => 'pp'.$participantId.'_session'.$session->getNumber().'_' . date('Y-m-d'),
    ];

    $exportService = $this->get('bazookas.export.service.ExportService');

    if (!$exportService instanceof ExportServiceInterface) {
      throw new \Exception('Invalid export service, must implement ' . ExportServiceInterface::class);
    }

    return $exportService->export($request, $config);
  }
```

### The export config

This example contains all of the required fields that are necessary in the export config.
- `sheets`: an array of sheets
- `fileName`: the name of the export file

#### Configuring sheets
The most important part of the config is the array with sheets. Each sheet requires an entity, an array of fields and a query-builder.
- `entity`: the entity of which you want to export data
- `fields`: an array containing the fields that need to be displayed on the export. Each element in the array is configured as `field` => `title`, where the title will be displayed at the top of the column
- `queryBuilder`: the query-builder that will be used to select the data to be displayed. This must contain an actual QueryBuilder instance, not the result of the query!

#### Export formats
There are two possible formats to export to: `.xlsx` and `.csv`. By default the export file will be `.csv`. To change the export format, you can add the `format` option to your export config.

Example:

```php
    $config = [
      'sheets' => [
        [
          'entity' => SalesOrder::class,
          'fields' => [
            ...
          ],
          'queryBuilder' => $salesOrderRepo->createSalesOrderExportQuery($from, $to)
        ]
      ],
      'fileName' => $fileName,
      'format' => ExportService::TYPE_XLSX,
    ];
```
