<?php
namespace Bazookas\GeneratorBundle\Command;

use Bazookas\CommonBundle\Entity\Base\BaseUser;
use Symfony\Bundle\FrameworkBundle\Command\ContainerAwareCommand;
use Symfony\Component\Console\Input\InputArgument;
use Symfony\Component\Console\Input\InputOption;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Output\OutputInterface;
use Symfony\Component\Console\Question\Question;

class CreateEntityInstanceCommand extends ContainerAwareCommand
{
  protected function configure()
  {
    $this
      ->setName('bazookas:create:entity-instance')
      ->setDescription('Creates a new client')
      ->addOption(
        'entity-class',
        null,
        InputOption::VALUE_REQUIRED,
        'Select which entity class you would like to use.',
        null
      )
      ->setHelp(
        "<<<EOT
          The <info>%command.name%</info>command creates a new Entity instance.
          Providing data to the actual properties is done through a wizard.
          <info>php %command.full_name% [--entity-class=...]</info>EOT"
      );
  }

  protected function execute(InputInterface $input, OutputInterface $output)
  {
    // NOTE: some user specific generation happens here, but it should be possible to generate other entity instances with this
    $entityClass = $input->getOption('entity-class');
    $helper = $this->getHelper('question');

    $container = $this->getContainer();
    /** @var \Doctrine\ORM\EntityManagerInterface $entityManager */
    $entityManager = $container->get('doctrine.orm.entity_manager');

    /** @var \Doctrine\ORM\Mapping\ClassMetadata $meta */
    $meta = $entityManager->getClassMetadata($entityClass);
    $reflectionClass = $meta->getReflectionClass();
    $constructorParameters = [];
    $method = $reflectionClass->getMethod('__construct');

    // create a salt, as seen in FOS\UserBundle\Util\PasswordUpdater
    $salt = rtrim(str_replace('+', '.', base64_encode(random_bytes(32))), '=');

    foreach ($method->getParameters() as $parameter) {
      // user specific
      if ($parameter->getName() === 'salt') {
        $constructorParameters[] = $salt;
      }
      else {
        $question = new Question('Please provide constructor param '.$parameter->getName().': ');
        $constructorParameters[] = $helper->ask($input, $output, $question);
      }
    }

    // create an instance
    $instance = $reflectionClass->newInstanceArgs($constructorParameters);
    $output->writeln('<comment>User instance created, now please fill in the details</comment>');

    // user specific
    if ($instance instanceof BaseUser) {
      $encoderFactory = $this->getContainer()->get('security.encoder_factory');
      /** @var \Symfony\Component\Security\Core\Encoder\PasswordEncoderInterface $encoder */
      $encoder = $encoderFactory->getEncoder($instance);

      $instance->setPassword(
        $encoder->encodePassword(
          $instance->getPassword(),
          $instance->getSalt()
        )
      );
      $output->writeln('<comment>Password encoded</comment>');
    }

    // loop over user fields and fill them
    // ignore certain fields, user specific
    $ignoreFields = [
      'setAccessTokens',
      'setRefreshTokens',
      'setRoles',
      'setPassword',
      'setPlainPassword',
      'setSalt',
      'setUsername',
      'setId',
      'setCreated',
      'setModified',
    ];
    foreach ($reflectionClass->getMethods() as $method) {
      // only try working with public setters
      $name = $method->getName();
      if ($method->isPublic() && strpos($name, 'set') === 0 && !in_array($name, $ignoreFields)) {
        $question = new Question(substr($name, 3).': ');
        $answer = $helper->ask($input, $output, $question);

        // TODO fix this hard coded birthdate check somehow
        if ($name === 'setBirthDate') {
          $answer = new \DateTime($answer, new \DateTimeZone('UTC'));
        }

        if (!empty($answer)) {
          $instance->{$name}($answer);
        }
      }
    }

    // all done
    $output->writeln('<comment>All done, trying to persist</comment>');
    $entityManager->persist($instance);
    $entityManager->flush();

    $output->writeln('<info>Great, no errors!</info> <comment>Entity was created and persisted to database.</comment>');
  }
}
