<?php

namespace Bazookas\GeneratorBundle\Command;

use Symfony\Bundle\FrameworkBundle\Command\ContainerAwareCommand;
use Symfony\Component\Console\Input\InputArgument;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Input\InputOption;
use Symfony\Component\Console\Output\OutputInterface;
use Symfony\Component\Console\Question\ConfirmationQuestion;
use Symfony\Component\Console\Question\Question;

use Sensio\Bundle\GeneratorBundle\Command\GeneratorCommand;
use Sensio\Bundle\GeneratorBundle\Command\GenerateControllerCommand;
use Bazookas\GeneratorBundle\Generator\AdminControllerGenerator;
use Sensio\Bundle\GeneratorBundle\Command\Validators;
use Doctrine\Bundle\DoctrineBundle\Mapping\DisconnectedMetadataFactory;

class GenerateAdminControllerCommand extends GenerateControllerCommand
{
    public function configure()
    {
        $this
          ->setName('bazookas:generate:admin-controller')
          ->setDescription('Generate an admin controller for use with the BazookasAdminBundle')
          ->setDefinition(array(
            new InputOption('controller', '', InputOption::VALUE_REQUIRED, 'The name of the controller to create'),
            new InputOption('route', '', InputOption::VALUE_REQUIRED, 'The route to use for this admin controller'),
            new InputOption('entity', '', InputOption::VALUE_REQUIRED, 'The entity to use for the admin controller (shortcut notation)'),
          ))
        ;
    }

    public function execute(InputInterface $input, OutputInterface $output)
    {
      $questionHelper = $this->getQuestionHelper();

      // ask user confirmation
      if ($input->isInteractive()) {
          $question = new ConfirmationQuestion($questionHelper->getQuestion('Do you confirm generation', 'yes', '?'), true);
          if (!$questionHelper->ask($input, $output, $question)) {
              $output->writeln('<error>Command aborted</error>');

              return 1;
          }
      }

      // validation
      if (null === $input->getOption('controller')) {
          throw new \RuntimeException('The controller option must be provided.');
      }

      /**********************
       * get the controller *
       **********************/
      list($bundle, $controller) = $this->parseShortcutNotation($input->getOption('controller'));
      if (is_string($bundle)) {
          $bundle = Validators::validateBundleName($bundle);

          try {
              $bundle = $this->getContainer()->get('kernel')->getBundle($bundle);
          } catch (\Exception $e) {
              $output->writeln(sprintf('<bg=red>Bundle "%s" does not exist.</>', $bundle));
          }
      }

      /******************
       * get the entity *
       ******************/
      $entityShortcut = Validators::validateEntityName($input->getOption('entity'));
      list($entityBundle, $entity) = $this->parseShortcutNotation($entityShortcut);

      $entityClass = $this->getContainer()->get('doctrine')->getAliasNamespace($entityBundle).'\\'.$entity;
      $metadata = $this->getEntityMetadata($entityClass);
      $entityBundle = $this->getApplication()->getKernel()->getBundle($entityBundle);

      /***********
       * EXECUTE *
       ***********/
      $questionHelper->writeSection($output, 'Controller generation');
      $generator = $this->getGenerator($bundle);
      $generator->generate(
        $bundle,
        $controller,
        'yml',
        $input->getOption('route'),
        $entity,
        $metadata[0]
      );
      $output->writeln('Generating the bundle code: <info>OK</info>');

      $questionHelper->writeGeneratorSummary($output, array());
    }


    public function interact(InputInterface $input, OutputInterface $output)
    {
        $questionHelper = $this->getQuestionHelper();
        $questionHelper->writeSection($output, 'Welcome to the BazookasAdminController generator');

        // namespace
        $output->writeln(array(
            'First, you need to give the controller name you want to generate.',
            'You must use the shortcut notation like <comment>AcmeBlogBundle:Post</comment>',
            '<info>NOTE:</info> "AdminController" is automatically appended to the end of the controller name',
            ' ',
        ));

        while (true) {
            $question = new Question($questionHelper->getQuestion('Controller name', $input->getOption('controller')), $input->getOption('controller'));
            $question->setValidator(array('Sensio\Bundle\GeneratorBundle\Command\Validators', 'validateControllerName'));
            // append Admin to the controller name
            $controller = $questionHelper->ask($input, $output, $question) . 'Admin';
            list($bundle, $controller) = $this->parseShortcutNotation($controller);

            try {
                $b = $this->getContainer()->get('kernel')->getBundle($bundle);

                if (!file_exists($b->getPath().'/Controller/'.$controller.'Controller.php')) {
                    break;
                }

                $output->writeln(sprintf('<bg=red>Controller "%s:%s" already exists.</>', $bundle, $controller));
            } catch (\Exception $e) {
                $output->writeln(sprintf('<bg=red>Bundle "%s" does not exist.</>', $bundle));
            }
        }
        $input->setOption('controller', $bundle.':'.$controller);

        // namespace
        $output->writeln(array(
            'now define your route, will be written in the bundle routing files.',
        ));
        $route = $this->askQuestion($input, $output, 'route', 'admin/route');


        $output->writeln(array(
            'which entity should be used',
        ));
        $entity = $this->askQuestion($input, $output, 'entity', 'Bundle:Entity');

        // NOTE: for now only yml format is supported for controller generation, see GenerateControllerCommand for implementation of other formats
        $routeFormat = 'yml';
        // $input->setOption('route-format', $routeFormat);

        // summary
        $output->writeln(array(
            '',
            $this->getHelper('formatter')->formatBlock('Summary before generation', 'bg=blue;fg-white', true),
            '',
            sprintf('You are going to generate a "<info>%s:%s</info>" controller', $bundle, $controller),
            sprintf('using the "<info>%s</info>" format for the routing', $routeFormat),
            sprintf('using the route <info>%s</info>', $route),
            sprintf('for entity <info>%s</info>', $entity),
        ));
    }

    protected function askQuestion($input, $output, $parameterName, $defaultValue)
    {
      $questionHelper = $this->getQuestionHelper();
      $defaultValue = (null !== $input->getOption($parameterName) ? $input->getOption($parameterName) : $defaultValue);

      $question = new Question($questionHelper->getQuestion("$parameterName:", $defaultValue), $defaultValue);
      $result = $questionHelper->ask($input, $output, $question);
      $input->setOption($parameterName, $result);

      return $result;
    }

    protected function getEntityMetadata($entity)
    {
      $factory = new DisconnectedMetadataFactory($this->getContainer()->get('doctrine'));

      return $factory->getClassMetadata($entity)->getMetadata();
    }


    protected function createGenerator()
    {
      return new AdminControllerGenerator($this->getContainer()->get('filesystem'));
    }

}
